<?php
/**
 * DBCart 商城系统
 *
 * ==========================================================================
 * @link      https://www.dbcart.cn/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   https://www.dbcart.cn/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Extend\Form;

use Laminas\Form\Form;

class OnlineServiceForm extends Form
{
    public function __construct($name = 'online-service-form', $options = [])
    {
        parent::__construct($name, $options);
        $this->setAttribute('method', 'post');
        $this->setAttribute('class', 'form-horizontal');

        $this->addElements();
        $this->addInputFilter();
    }

    protected function addElements()
    {
        foreach (SHOP_LANGUAGE as $language) {
            $this->add([
                'type'  => 'text',
                'name'  => 'onlineTitle1'.$language,
                'attributes'    => [
                    'id'            => 'onlineTitle1'.$language,
                    'class'         => 'form-control'
                ]
            ]);

            $this->add([
                'type'  => 'text',
                'name'  => 'onlineTitle2'.$language,
                'attributes'    => [
                    'id'            => 'onlineTitle2'.$language,
                    'class'         => 'form-control'
                ]
            ]);

            $this->add([
                'type'  => 'text',
                'name'  => 'onlineTitle3'.$language,
                'attributes'    => [
                    'id'            => 'onlineTitle3'.$language,
                    'class'         => 'form-control'
                ]
            ]);

            $this->add([
                'type'  => 'text',
                'name'  => 'onlineTitle4'.$language,
                'attributes'    => [
                    'id'            => 'onlineTitle4'.$language,
                    'class'         => 'form-control'
                ]
            ]);

            $this->add([
                'type'  => 'text',
                'name'  => 'onlineTitle5'.$language,
                'attributes'    => [
                    'id'            => 'onlineTitle5'.$language,
                    'class'         => 'form-control'
                ]
            ]);
        }

        $this->add([
            'type'  => 'select',
            'name'  => 'onlineType1',
            'attributes'    => [
                'id'            => 'onlineType1',
                'class'         => 'form-control'
            ]
        ]);
        $this->add([
            'type'  => 'text',
            'name'  => 'onlineAccount1',
            'attributes'    => [
                'id'            => 'onlineAccount1',
                'class'         => 'form-control'
            ]
        ]);
        $this->add([
            'type'  => 'multiCheckbox',
            'name'  => 'onlineDisplayRange1',
            'options' => [
                'label_attributes' => [
                    'class' => 'checkbox-inline'
                ]
            ]
        ]);

        $this->add([
            'type'  => 'multiCheckbox',
            'name'  => 'onlineClient',
            'options' => [
                'label_attributes' => [
                    'class' => 'checkbox-inline'
                ]
            ]
        ]);

        $this->add([
            'type'  => 'select',
            'name'  => 'onlineType2',
            'attributes'    => [
                'id'            => 'onlineType2',
                'class'         => 'form-control'
            ]
        ]);
        $this->add([
            'type'  => 'text',
            'name'  => 'onlineAccount2',
            'attributes'    => [
                'id'            => 'onlineAccount2',
                'class'         => 'form-control'
            ]
        ]);
        $this->add([
            'type'  => 'multiCheckbox',
            'name'  => 'onlineDisplayRange2',
            'options' => [
                'label_attributes' => [
                    'class' => 'checkbox-inline'
                ]
            ]
        ]);

        $this->add([
            'type'  => 'select',
            'name'  => 'onlineType3',
            'attributes'    => [
                'id'            => 'onlineType3',
                'class'         => 'form-control'
            ]
        ]);
        $this->add([
            'type'  => 'text',
            'name'  => 'onlineAccount3',
            'attributes'    => [
                'id'            => 'onlineAccount3',
                'class'         => 'form-control'
            ]
        ]);
        $this->add([
            'type'  => 'multiCheckbox',
            'name'  => 'onlineDisplayRange3',
            'options' => [
                'label_attributes' => [
                    'class' => 'checkbox-inline'
                ]
            ]
        ]);

        $this->add([
            'type'  => 'select',
            'name'  => 'onlineType4',
            'attributes'    => [
                'id'            => 'onlineType4',
                'class'         => 'form-control'
            ]
        ]);
        $this->add([
            'type'  => 'text',
            'name'  => 'onlineAccount4',
            'attributes'    => [
                'id'            => 'onlineAccount4',
                'class'         => 'form-control'
            ]
        ]);
        $this->add([
            'type'  => 'multiCheckbox',
            'name'  => 'onlineDisplayRange4',
            'options' => [
                'label_attributes' => [
                    'class' => 'checkbox-inline'
                ]
            ]
        ]);

        $this->add([
            'type'  => 'select',
            'name'  => 'onlineType5',
            'attributes'    => [
                'id'            => 'onlineType5',
                'class'         => 'form-control'
            ]
        ]);
        $this->add([
            'type'  => 'text',
            'name'  => 'onlineAccount5',
            'attributes'    => [
                'id'            => 'onlineAccount5',
                'class'         => 'form-control'
            ]
        ]);
        $this->add([
            'type'  => 'multiCheckbox',
            'name'  => 'onlineDisplayRange5',
            'options' => [
                'label_attributes' => [
                    'class' => 'checkbox-inline'
                ]
            ]
        ]);
    }

    protected function addInputFilter()
    {
        $inputFilter = $this->getInputFilter();

        foreach (SHOP_LANGUAGE as $language) {
            $inputFilter->add([
                'name'      => 'onlineTitle1'.$language,
                'required'  => false,
                'filters'   => [
                    ['name' => 'StringTrim'],
                    ['name' => 'StripTags']
                ]
            ]);

            $inputFilter->add([
                'name'      => 'onlineTitle2'.$language,
                'required'  => false,
                'filters'   => [
                    ['name' => 'StringTrim'],
                    ['name' => 'StripTags']
                ]
            ]);

            $inputFilter->add([
                'name'      => 'onlineTitle3'.$language,
                'required'  => false,
                'filters'   => [
                    ['name' => 'StringTrim'],
                    ['name' => 'StripTags']
                ]
            ]);

            $inputFilter->add([
                'name'      => 'onlineTitle4'.$language,
                'required'  => false,
                'filters'   => [
                    ['name' => 'StringTrim'],
                    ['name' => 'StripTags']
                ]
            ]);

            $inputFilter->add([
                'name'      => 'onlineTitle5'.$language,
                'required'  => false,
                'filters'   => [
                    ['name' => 'StringTrim'],
                    ['name' => 'StripTags']
                ]
            ]);
        }

        $inputFilter->add([
            'name'      => 'onlineType1',
            'required'  => false
        ]);
        $inputFilter->add([
            'name'      => 'onlineAccount1',
            'required'  => false,
            'filters'   => [
                ['name' => 'StringTrim'],
                ['name' => 'StripTags']
            ]
        ]);
        $inputFilter->add([
            'name'      => 'onlineDisplayRange1',
            'required'  => false
        ]);
        $inputFilter->add([
            'name'      => 'onlineClient',
            'required'  => false
        ]);

        $inputFilter->add([
            'name'      => 'onlineType2',
            'required'  => false
        ]);
        $inputFilter->add([
            'name'      => 'onlineAccount2',
            'required'  => false,
            'filters'   => [
                ['name' => 'StringTrim'],
                ['name' => 'StripTags']
            ]
        ]);
        $inputFilter->add([
            'name'      => 'onlineDisplayRange2',
            'required'  => false
        ]);

        $inputFilter->add([
            'name'      => 'onlineType3',
            'required'  => false
        ]);
        $inputFilter->add([
            'name'      => 'onlineAccount3',
            'required'  => false,
            'filters'   => [
                ['name' => 'StringTrim'],
                ['name' => 'StripTags']
            ]
        ]);
        $inputFilter->add([
            'name'      => 'onlineDisplayRange3',
            'required'  => false
        ]);

        $inputFilter->add([
            'name'      => 'onlineType4',
            'required'  => false
        ]);
        $inputFilter->add([
            'name'      => 'onlineAccount4',
            'required'  => false,
            'filters'   => [
                ['name' => 'StringTrim'],
                ['name' => 'StripTags']
            ]
        ]);
        $inputFilter->add([
            'name'      => 'onlineDisplayRange4',
            'required'  => false
        ]);

        $inputFilter->add([
            'name'      => 'onlineType5',
            'required'  => false
        ]);
        $inputFilter->add([
            'name'      => 'onlineAccount5',
            'required'  => false,
            'filters'   => [
                ['name' => 'StringTrim'],
                ['name' => 'StripTags']
            ]
        ]);
        $inputFilter->add([
            'name'      => 'onlineDisplayRange5',
            'required'  => false
        ]);

    }
}